/* BEGIN COPYRIGHT */
/*************************************************************************
 * Copyright (C) 2003-2013
 * Raytheon Company
 * Unpublished Work
 * ALL RIGHTS RESERVED
 *
 * This software was developed pursuant to
 * Contract Number F04701-02-C-0502 with the US Government
 * under subcontract number 47974DGM2S. The US Government's rights
 * in and to this copyrighted software are as specified in DFAR
 * 252.227-7014 which was made part of the above contract.
 *
 * Portions of this work are the copyrighted work of Raytheon.
 * However, other entities may own copyrights in this work. Therefore,
 * the recipient should not imply that Raytheon is the only copyright
 * owner in this work.
 *************************************************************************/
/* END COPYRIGHT */

/* BEGIN GOVERNMENT_RIGHTS */
/***********************************************************************
 * UNLIMITED RIGHTS
 * This material may be reproduced by or on behalf of the
 * U.S. Government pursuant to the copyright license under the DFAR
 * 252.227-7014 which was made part of the above contract.
 ***********************************************************************/
/* END GOVERNMENT_RIGHTS */

/* BEGIN ITAR */
/* END ITAR */

/* BEGIN CATEGORY */
/**************************************************************************
* CATEGORY: 1 (see SEN for definitions)
*
* (ProCmnMetadata.h)
*
* Refer to Software Standards and Practices Manual (SSPM) for
* coding standards.
*
**************************************************************************/
/* END CATEGORY */

/**************************************************************************
*
* NAME: ProCmnMetadata
*
* DESCRIPTION: The ProCmnMetadata.h file contains all of the metadata names to
*              be used in populating metadata.  All PRO metadata names should
*              be referenced here and added to the ProCmnMetadata.cpp file.
*
* REFERENCES:
* none
*
* LIMITATIONS:
* none
*
* REVISION/EVENT HISTORY:
* DATE        PR#      AUTHOR            Build    DESCRIPTION
* ---------   ---      ------            -----    -----------
* 23JAN2004            R. Arbeiter       1.2      Initial Version
* 22JUN2004   2564     M. Hoover         1.2      Added metadata in accordance
* 20SEP2004            B. Henderson      1.3      Add missing metadata strings
* 04APR2005            M. Hoover         1.3      Add DatasetLock
* 21APR2005            M. Hoover         1.3      Added Rev J metadata
* 24JUN2005            B. Henderson      1.3      Add SDR metadata structure
* 17NOV2005            B. Henderson      1.4      Add UpdateDateTime metadata
* 09MAY2006            D. Stuhmer        1.4      extern all strings with
*                                                 initialization in .cpp to
*                                                 address TOC overflow.
* 26JUL2006  11920     D. Stuhmer        1.4      First draft of changes to
*                                                 metadata for ECR-IDP-R479.
* 04AUG2006  12044     D. Stuhmer        1.4      Platform_Short_Name is no
*                                                 longer a group.
* 07AUG2006            C. Erkes          1.5      Added granule-level quality
*                                                 flag metadata
*
* 13SEP2006  11833     D. Stuhmer        1.4      Fixed implementation of
*                                                 N_ANC_TYPE metadata and
*                                                 removed unused version
*                                                 strings and cleaned up
*                                                 comments.
* 08NOV2006            D. Hrivnak        1.5      Added GIP_CONSTELLATION
* 12JAN2007            B. Henderson      1.5      Added granule level quality
*                                                 flag metadata for Aerosols
* 30JAN2007            D. Stuhmer        1.5      Removed unused strings,
*                                                 added new B1.5 strings,
*                                                 updated code headers,
*                                                 updated comments for 1.5.
* 15MAR2007            B. Henderson      1.5      Updated for CMN review rework
* 30MAR2007            B. Henderson      1.5      Added new metadata strings
* 03MAY2007            J. Marion         1.5      Added granule level quality
*                                                 flag metadata for cloud edrs
* 07MAY2007            J. Idol           1.5      DQN'S for VIIRS SDR
* 20JUN2007            B. Henderson      1.5      Added CrIS SDR DQN strings
* 23JUN2007            B. Hurley         1.5      Added granule level quality
*                                                 flag metadata for ATMS.
* 12MAY2008            J. Idol           1.5X1    Added constants for N_LEOA
*                                                 metadata
* 26FEB2008            B. Henderson      1.5x1    Added new quality summary
*                                                 strings for Net Heat Flux
* 26FEB2008            M. Howatt         1.5x1    Added Synoptic
* 11APR2008            E. Johnson        1.5x1    Added Forecast
* 17SEP2008            J. Gibbs          1.5x1    added missing quality
*                                                 metadata names
*                                                 Implement Tech Memo:
*                                                 Build 1.5- DQNs 2006.510.009
* 18NOV2008  18858     W. Dare           1.5x1    Added combined Surface Albedo
*                                                 DQN/Metadata strings
* 20NOV2008            K. Bisanz         1.5x1    Added AuxID and AuxVersion
* 03DEC2008  18898     J. Gibbs          1.5x1    removed the Shell metadata
*                                                 string
* 06JAN2009  017744    R. Ferrel         1.5x1    Add UNKNOWN and sensor modes.
* 29JAN2009  19494     N. Emmert         1.5x1    Added NP and TC Qual Summary
*                                                 strings.
* 30MAR2009  20081     D. Stuhmer        PostX    Implement N/S/E/W bounding box
*                                                 metadata.
* 14APR2009  20087     J. Gibbs          PostX    Added the geo metadata items
* 06MAY2009  20390     F. DeLuca         PostX    Ensure that all non-band specific
*                                                 VIIRS SDR products and all non-VIIRS
*                                                 SDR products have a Band_ID metadata
*                                                 element with the value "N/A" as per
*                                                 the CDFCB-X, vol V.
* 09DEC2009  21752     J. Gibbs          SenChar  Added the Aerosol APSP
*                                                 Quality Summary
* 16MAR2010  22881     B. Kowal          SenChar  Added the Aerosol APSP
*                                                 Exclusion Summary
* 17MAR2010  22880     B. Kowal          SenChar  Added the Suspended Matter Typing
*                                                 Exclusion Summary
* 18MAR2010  22844     J. Gibbs          SenChar  Modified to add OMPS TC/NP 
* 01APR2010  22903     Y. Liu            SenChar  Added the Suspended Matter Typing
*                                                 Quality Summary
* 13DEC2011  19883     S. Joo            1.5.7    Removed Row, Columns and
*                                                 Levels from MetaDataTypes
* 04OCT2012  31669     W. Dare           Mx7      Added InputTile metadata for
*                                                 GridIP-Gran products
* 06JUN2013  34491     G. Pachl          1.5.08   Added RSB Index metadata.
*
* NOTES (MISCELLANEOUS) SECTION:
*  Implemented TM2007.510.0044
*  Implemented SDG187
*
* --------- -------- -----------------------------------
* <header review information>
*
**************************************************************************/

#ifndef _PROCMNMETADATA_H
#define _PROCMNMETADATA_H

#include <string>
#include <ProCmnDefs.h>

//
// String constants for the Legal metadata names.  (for B1.5)
// The Metadata names are in alphabetical order according to the DPIS ICD.
// The Metadata names that are a DMS Group item (listed as Repeating) are
// now listed first (in the Name Column of the DPIS) and marked as a Group.
// The associated metadata attribute name is now listed after the group item
// and indented (this is the name listed in the Comment Column of the DPIS).
// The order of this list matches the order in ProCmnMetadata.cpp.
//
extern const std::string ALGORITHMVERSION;
extern const std::string ANCFILENAME;
extern const std::string    N_ANC_FILENAME;               // Group
extern const std::string ASCENDING_DESCENDING_INDICATOR;
extern const std::string AUXID;
extern const std::string AUXFILENAME;
extern const std::string    N_AUX_FILENAME;               // Group
extern const std::string AUXVERSION;
extern const std::string BAND_ID;

// ADL only
extern const std::string BYTEORDER;

extern const std::string BEGINNINGORBITNUMBER;
extern const std::string CLOUD_COVER;
extern const std::string CREATIONDATETIME;
extern const std::string DATASETLOCK;
extern const std::string DIMNAMES;                        // Group
extern const std::string    NAMES;
extern const std::string DIMVALUES;                       // Group
extern const std::string    VALUES;
extern const std::string DIMTYPES;                        // Group
extern const std::string    TYPES;
extern const std::string EFFECTIVITY;
extern const std::string FORECAST;
extern const std::string GRINGLATITUDE;                   // Group
extern const std::string GRINGLONGITUDE;                  // Group
extern const std::string INPUTPROD;
extern const std::string    N_INPUT_PROD;                 // Group
extern const std::string INPUTTILE;
extern const std::string    N_INPUT_TILE;
extern const std::string INSTRUMENTFLIGHTSW;
extern const std::string    N_INSTRUMENT_FLIGHT_SW_VERSION; // Group
extern const std::string INSTRUMENT_SHORT_NAME;
extern const std::string MISSION_NAME;
extern const std::string N_ANC_PRODUCT_NAME;
extern const std::string N_ANC_TYPE;
extern const std::string N_ANC_TYPE_TASKED;
extern const std::string N_COLLECTION_SHORT_NAME;
extern const std::string N_DATA_SET_SOURCE;
extern const std::string N_DATA_SET_TYPE_TAG;
extern const std::string N_DAY_NIGHT_FLAG;
extern const std::string N_GRACEFUL_DEGRADATION;
extern const std::string N_GRANULE_ID;
extern const std::string N_GRANULE_VERSION;
extern const std::string N_LEOA_FLAG;
extern const std::string N_LEOA_SET_VALUE;
extern const std::string N_LEOA_NOTSET_VALUE;
extern const std::string N_NADIR_LATITUDE_MAX;
extern const std::string N_NADIR_LATITUDE_MIN;
extern const std::string N_NADIR_LONGITUDE_MAX;
extern const std::string N_NADIR_LONGITUDE_MIN;
extern const std::string N_NUMBER_OF_SCANS;
extern const std::string N_PERCENT_ERRONEOUS_DATA;
extern const std::string N_PERCENT_MISSING_DATA;
extern const std::string N_PERCENT_NA_DATA;
extern const std::string N_PROCESSING_DOMAIN;
extern const std::string N_RSB_INDEX;
extern const std::string N_SATELLITE_LOCAL_AZIMUTH_ANGLE_MAX;
extern const std::string N_SATELLITE_LOCAL_AZIMUTH_ANGLE_MIN;
extern const std::string N_SATELLITE_LOCAL_ZENITH_ANGLE_MAX;
extern const std::string N_SATELLITE_LOCAL_ZENITH_ANGLE_MIN;
extern const std::string N_SOLAR_AZIMUTH_ANGLE_MAX;
extern const std::string N_SOLAR_AZIMUTH_ANGLE_MIN;
extern const std::string N_SOLAR_ZENITH_ANGLE_MAX;
extern const std::string N_SOLAR_ZENITH_ANGLE_MIN;
extern const std::string N_SPACECRAFT_MANEUVER;
extern const std::string N_SOFTWARE_VERSION;
extern const std::string NORTH_BOUNDING_COORDINATE;
extern const std::string SOUTH_BOUNDING_COORDINATE;
extern const std::string EAST_BOUNDING_COORDINATE;
extern const std::string WEST_BOUNDING_COORDINATE;
extern const std::string OPERATIONAL_MODE;
extern const std::string OBSERVEDDATETIME;
extern const std::string PLATFORM_SHORT_NAME;
extern const std::string QUALITYNAMES;
extern const std::string    N_QUALITY_SUMMARY_NAMES;      // Group
extern const std::string QUALITYVALUES;
extern const std::string    N_QUALITY_SUMMARY_VALUES;     // Group
extern const std::string RANGEDATETIME;
extern const std::string SHELL_STATUS;
extern const std::string STATICANC;
extern const std::string SYNOPTIC;
extern const std::string TILEID;                          // Group
extern const std::string UPDATEDATETIME;

//
// The string constants below are used as default values for various
// metadata names
//
extern const std::string PROCMN_INVALID_ALGOVERSION;
extern const std::string PROCMN_YES_STRING;
extern const std::string PROCMN_NO_STRING;
extern const std::string PROCMN_BE_STRING;
extern const std::string PROCMN_LE_STRING;
extern const std::string PROCMN_OFFICIAL_STRING;
extern const std::string PROCMN_SUBSTITUTE_STRING;
extern const std::string PROCMN_DAY;
extern const std::string PROCMN_NIGHT;
extern const std::string PROCMN_BOTH;
extern const std::string PROCMN_INVALID_MANEUVER;
extern const std::string PROCMN_SCIENCE;
extern const std::string PROCMN_SCIENCE_CALIBRATION;
extern const std::string PROCMN_ORBIT_ADJUST;
extern const std::string PROCMN_UNKNOWN;
extern const std::string PROCMN_OPERATIONAL;
extern const std::string PROCMN_DIAGNOSTIC;
extern const std::string PROCMN_OPER_AND_DIAG;
extern const std::string PROCMN_CALIBRATION;
extern const std::string PROCMN_NOT_APPLICABLE;

//
// Values to use for setting N_Spacecraft_Maneuver metadata and
// for determining the S/C Mode and the Operational Mode for
// the spacecraft.
//
extern const std::string NORMAL_OPERATIONS;
extern const std::string CAL_MANEUVER;
extern const std::string ORBIT_CORR_MANEUVER;
extern const std::string UNKNOWN_MANEUVER;

/*
 * Values to use for setting sensor modes metadata.
 */
extern const std::string MODE_OPERATIONAL;   //= "Operational";
extern const std::string MODE_DIAGNOSTIC;    //= "Diagnostic";
extern const std::string MODE_OPER_AND_DIAG; //= "Operational and Diagnostic";
extern const std::string MODE_CALIBRATION;   //= "Calibration";
extern const std::string MODE_UNKNOWN;       //= "Unknown";

//
// Structure to define the xDR output metadata
//
struct ProCmnMetaDataType {
    std::string bandID;                // Band_ID
};

//
// The string constants below are used to define granule level quality
// flags in metadata
//

// SST EDR
extern const std::string SST_FAV_SKIN_QUAL;
extern const std::string SST_FAV_BULK_QUAL;
extern const std::string SST_UNFAV_SKIN_QUAL;
extern const std::string SST_UNFAV_BULK_QUAL;
extern const std::string SST_EXCL_SKIN;
extern const std::string SST_EXCL_BULK;
extern const std::string SST_SKIN_RANGE;
extern const std::string SST_BULK_RANGE;

// CrIMSS EDR
extern const std::string CRIMSS_PRODUCT_YIELD;
extern const std::string CRIMSS_CRIS_INPUT_QUAL;
extern const std::string CRIMSS_ATMS_INPUT_QUAL;
extern const std::string CRIS_IR_OZONE_RET_QUAL;

// Ice Age(Sea Ice Char) EDR
extern const std::string ICE_AGE_RET_HIGH_QUAL;
extern const std::string ICE_AGE_EXCL_SUMMARY;
extern const std::string ICE_AGE_EXCL_NO_OCEAN;
extern const std::string ICE_AGE_OUT_OF_RANGE;

// Surface Type EDR
extern const std::string ST_QUALITY;
extern const std::string ST_EXCL_SUMMARY;

// OCC EDR
extern const std::string OCC_RET_M1_HIGH_QUAL;
extern const std::string OCC_RET_M2_HIGH_QUAL;
extern const std::string OCC_RET_M3_HIGH_QUAL;
extern const std::string OCC_RET_M4_HIGH_QUAL;
extern const std::string OCC_RET_M5_HIGH_QUAL;
extern const std::string OCC_CHL_CONC_HIGH_QUAL;
extern const std::string OCC_IOPA_M1_HIGH_QUAL;
extern const std::string OCC_IOPA_M2_HIGH_QUAL;
extern const std::string OCC_IOPA_M3_HIGH_QUAL;
extern const std::string OCC_IOPA_M4_HIGH_QUAL;
extern const std::string OCC_IOPA_M5_HIGH_QUAL;
extern const std::string OCC_IOPS_M1_HIGH_QUAL;
extern const std::string OCC_IOPS_M2_HIGH_QUAL;
extern const std::string OCC_IOPS_M3_HIGH_QUAL;
extern const std::string OCC_IOPS_M4_HIGH_QUAL;
extern const std::string OCC_IOPS_M5_HIGH_QUAL;
extern const std::string OCC_EXCL_SUM;
extern const std::string OCC_OUT_RANGE;
extern const std::string OCC_CHL_CONC_OUT_RANGE;
extern const std::string OCC_IOPA_OUT_RANGE;
extern const std::string OCC_IOPS_OUT_RANGE;
extern const std::string OCC_EXCL_NO_OCEAN;

// Active Fires EDR
extern const std::string AF_FIRE_CONF_HIGH;

// VI EDR
extern const std::string VI_EVI_QUAL;
extern const std::string VI_NDVI_QUAL;
extern const std::string VI_TOCNDVI_QUAL;
extern const std::string VI_EVI_EXCL_SUM;
extern const std::string VI_NDVI_EXCL_SUM;
extern const std::string VI_TOCNDVI_EXCL_SUM;
extern const std::string VI_EXCL_NO_LAND;

// Snow
extern const std::string SCD_MAP_QUAL;
extern const std::string SCD_MAP_EXCLUS;
extern const std::string SCD_FRAC_QUAL;
extern const std::string SCD_FRAC_DEGRED;
extern const std::string SCD_FRAC_EXCLUS;

// GIP
extern const std::string GIP_CONSTELLATION;

// Aerosol quality metadata strings
extern const std::string AEROS_AOT_QUALITY_SUMMARY;
extern const std::string AEROS_APSP_QUALITY_SUMMARY;
extern const std::string AEROS_AOT_EXCLUSION_SUMMARY;
extern const std::string AEROS_APSP_EXCLUSION_SUMMARY;
extern const std::string AEROS_NO_LAND;
extern const std::string AEROS_NO_OCEAN;

// VIIRS AEROS GEO
extern const std::string AEROS_PERCENT_MISSING_DATA;
extern const std::string AEROS_PERCENT_OUT_OF_BOUNDS;
extern const std::string AEROS_AUTOMATIC_QUALITY_FLAG;

// SM
extern const std::string SUSMAT_SUMMARY;
extern const std::string SUSMAT_TYPING_SUMMARY;
extern const std::string SUSMAT_DETECT_EXCLUSION;
extern const std::string SUSMAT_TYPING_EXCLUSION;

// CrIS SDR
extern const std::string CRIS_RDR_YIELD;
extern const std::string CRIS_SDR_YIELD;
extern const std::string CRIS_INVALID_RAD_CALI_YIELD;

// VIIRS SDR
extern const std::string VIIRS_SDR_QUALITY;
extern const std::string VIIRS_SDR_EXCLUSION;

// VIIRS SDR Geolocation
extern const std::string VIIRS_SDR_GEO_PER_MISSING_DATA;
extern const std::string VIIRS_SDR_GEO_PER_OUTOFBOUNDS;
extern const std::string VIIRS_SDR_GEO_AUTO_QUAL_FLAG;


///VIIRS GTM I Band and NCC
extern const std::string VIIRS_EDR_QUALITY;
extern const std::string VIIRS_EDR_RANGE;
extern const std::string VIIRS_EDR_SATURATED;

// GCE
extern const std::string CBH_EDR_QUALITY;
extern const std::string CBH_EDR_EXCLUSION;

extern const std::string CCL_EDR_QUALITY;
extern const std::string CCL_EDR_EXCLUSION;

extern const std::string CEPS_EDR_QUALITY;
extern const std::string CEPS_EDR_EXCLUSION;

extern const std::string COT_EDR_QUALITY;
extern const std::string COT_EDR_EXCLUSION;

extern const std::string CTH_EDR_QUALITY;
extern const std::string CTH_EDR_EXCLUSION;

extern const std::string CTP_EDR_QUALITY;
extern const std::string CTP_EDR_EXCLUSION;

extern const std::string CTT_EDR_QUALITY;
extern const std::string CTT_EDR_EXCLUSION;

// VIIRS CLOUDS AGGREGATE GEO
extern const std::string CLD_AGG_PERCENT_MISSING_DATA;
extern const std::string CLD_AGG_PERCENT_OUT_OF_BOUNDS;
extern const std::string CLD_AGG_AUTOMATIC_QUALITY_FLAG;

// ATMS
extern const std::string ATMS_SDR_QUALITY_SUMMARY;
extern const std::string ATMS_TDR_QUALITY_SUMMARY;

// VIIRS NHF
extern const std::string NHF_TOTAL_PRODUCT_QUALITY;
extern const std::string NHF_EXCLUSION_SUMMARY;
extern const std::string NHF_NO_OCEAN_COVERAGE;

// VIIRS NHF GEO
extern const std::string NHF_PERCENT_MISSING_DATA;
extern const std::string NHF_PERCENT_OUT_OF_BOUNDS;
extern const std::string NHF_AUTOMATIC_QUALITY_FLAG;

// VIIRS IST
extern const std::string IST_TOTAL_PRODUCT_QUALITY;
extern const std::string IST_SDR_INPUT_PRODUCT_QUALITY;
extern const std::string IST_EXCLUSION_SUMMARY;
extern const std::string IST_SUMMARY_RANGE_CHECK;
extern const std::string IST_IP_INPUT_DATA_QUALITY;
extern const std::string IST_VCM_INPUT_QUALITY;
extern const std::string IST_AOT_INPUT_QUALITY;
extern const std::string IST_NO_OCEAN_COVERAGE;
extern const std::string IST_NO_LAND_COVERAGE;

// VIIRS LST
extern const std::string LST_TOTAL_PRODUCT_QUALITY;
extern const std::string LST_SDR_INPUT_PRODUCT_QUALITY;
extern const std::string LST_EXCLUSION_SUMMARY;
extern const std::string LST_SUMMARY_RANGE_CHECK;
extern const std::string LST_ST_INPUT_DATA_QUALITY;
extern const std::string LST_VCM_INPUT_QUALITY;
extern const std::string LST_AOT_INPUT_QUALITY;
extern const std::string LST_NO_LAND_COVERAGE;

// VIIRS Land Surface Albedo
extern const std::string LSA_TOTAL_PRODUCT_QUALITY;
extern const std::string LSA_EXCLUSION_SUMMARY;

// VIIRS Combined Surface Albedo
extern const std::string SA_TOTAL_PRODUCT_QUALITY;
extern const std::string SA_EXCLUSION_SUMMARY;
extern const std::string SA_SUMMARY_RANGE_CHECK;
extern const std::string SA_NO_OCEAN_COVERAGE;
extern const std::string SA_NO_LAND_COVERAGE;

// OMPS Nadir Profile
extern const std::string OMPS_NP_QUALITY_SUMMARY;
extern const std::string OMPS_NP_INPUT_DATA_SUMMARY;

// Omps Total Column
extern const std::string OMPS_TC_QUALITY_SUMMARY;
extern const std::string OMPS_TC_EXCLUSION_SUMMARY;
extern const std::string OMPS_TC_INPUT_DATA_SUMMARY;

#if ADL_ENV
// Extra group only in ADL to make it easier to grab all inputs used for an
// output
extern const std::string ALL_INPUT_META_FILES;
extern const std::string ALL_INPUT_BINARY_FILES;
extern const std::string    N_INPUT_FILE_NAME;
#endif



//
// end of header file
//

#endif
