/* BEGIN COPYRIGHT */
/* END COPYRIGHT */
/* BEGIN GOVERNMENT_RIGHTS */
/***********************************************************************
 * UNLIMITED RIGHTS
 * This material may be reproduced by or on behalf of the
 * U.S. Government pursuant to the copyright license under the DFAR
 * 252.227-7014 which was made part of the above contract.
 ***********************************************************************/
/* END GOVERNMENT_RIGHTS */
/* BEGIN ITAR */
/* END ITAR */

/* BEGIN CATEGORY */
/************************************************************************** 
 * CATEGORY: 1 (see SEN for definitions)
 *
 * (ProEdrViirsVIDelegate.cpp)
 *
 * Refer to Software Standards and Practices Manual (SSPM) for 
 * coding standards.
 *
 **************************************************************************/
/* END CATEGORY */

/**************************************************************************
 * 
 * NAME: ProEdrViirsVIDelegate
 *
 * DESCRIPTION: 
 *
 *
 * USAGE: See header file for more information.
 *
 * Parameter         Type          I/O    Description
 * -----------       -----         ---    ----------------
 * See the method descriptions located in the .h file for details.
 *
 *
 * REFERENCES:

 *
 * LIMITATIONS: 
 * none
 *
 *
 * HISTORY:
 *   DATE:       PR#         AUTHOR      Description
 *   --------    ------      --------    -----------------
 *   21JAN2011               D. Doumakes Initial version
 *   03DEC2010   24551       J.DeLotelle Modified createQualityTests() to add
 *                                       qualityFillPtr argument to 
 *                                       ProCmnPercentQualityFlag call.
 *                                       This removes DNE pixels from the 
 *                                       denominator of summary calculations.
 *  11JAN2012    21415       W. Dare     Document memory deletion
 *
 * NOTES (MISCELLANEOUS) SECTION:
 * none
 *
 * --------- -------- -----------------------------------
 * <header review information>
 *
 **************************************************************************/

#include <ProEdrViirsVIDelegate.h>
#include <ProEdrViirsVI.h>
#include <ProCmnSDRItem.h>
#include <ProCmnEDROutputItem.h>
#include <ProCmnMethodAudit.h>

#include <ProCmnAuxDataItem.h>
#include <IngMsdThresholds_Structs.h>
#include <ProEdrViirsVI.h>
#include <VegIndex.h>
#include <ProEdrViirsVIGbl.h>
#include <ProCmnMetadata.h>
#include <ProCmnDqttTestId.h>
#include <ProCmnPercentQualityFlag.h>
#include <ProCmnLogger.h>
#include <ProCmnCalculateSummary.h>
#include <ProCmnViirsTrimTable.h>
#include <ProCmnShortNames.h>
#include <ProCmnQualityNotification.h>
#include <ProCmnDQNOutputItem.h>
#include <Typedefs.h>

//-----------------------------------------------------------------------------
// Constructor
//-----------------------------------------------------------------------------
ProEdrViirsVIDelegate::ProEdrViirsVIDelegate(ProCmnAlgorithm* derivedAlg) :
    derivedAlg_(reinterpret_cast<ProEdrViirsVI*>(derivedAlg)),
    identity_("ProEdrViirsVIDelegate"),
    myCallerID_("ProEdrViirsVIDelegate")
{
   // Allocate memory for doing exclusion summary
   // memory deleted in destructor
   exclusionSummaryDataPtr_ = new UInt8[M_VIIRS_SDR_ROWS * M_VIIRS_SDR_COLS];

   // Allocate memory for doing exclusion summary
   // memory deleted in destructor
   ndviExclusionSummaryDataPtr_ = new UInt8[I_VIIRS_SDR_ROWS * I_VIIRS_SDR_COLS];
   eviExclusionSummaryDataPtr_ = new UInt8[I_VIIRS_SDR_ROWS * I_VIIRS_SDR_COLS];
   tocndviExclusionSummaryDataPtr_ = new UInt8[I_VIIRS_SDR_ROWS * I_VIIRS_SDR_COLS];   
}

//-----------------------------------------------------------------------------
// Destructor
//-----------------------------------------------------------------------------
/*virtual*/ ProEdrViirsVIDelegate::~ProEdrViirsVIDelegate()
{
   // release exclusion summary memory if allocated
   if (exclusionSummaryDataPtr_ != 0)
   {
      delete [] exclusionSummaryDataPtr_;
      exclusionSummaryDataPtr_ = 0;
   }
   
   if (ndviExclusionSummaryDataPtr_ != 0)
   {
      delete [] ndviExclusionSummaryDataPtr_;
      ndviExclusionSummaryDataPtr_ = 0;
   }
   
   if (eviExclusionSummaryDataPtr_ != 0)
   {
      delete [] eviExclusionSummaryDataPtr_;
      eviExclusionSummaryDataPtr_ = 0;
   }
   
   if (tocndviExclusionSummaryDataPtr_ != 0)
   {
      delete [] tocndviExclusionSummaryDataPtr_;
      tocndviExclusionSummaryDataPtr_ = 0;
   }
}

//-----------------------------------------------------------------------------
// Create DQTT test objects for DQNs
//-----------------------------------------------------------------------------
/* virtual */ void ProEdrViirsVIDelegate::createQualityTests()
{
   // log the begining of the function call
   ProCmnMethodAudit methodAudit(
         this,
         "ProEdrViirsVIDelegate",
         "createQualityTests",
         derivedAlg_->getMethodContext());

   // Add the quality metadata names to the map.
   ProCmnEDROutputItem * vegIndexEdrPtr = 
         static_cast<ProCmnEDROutputItem*>(derivedAlg_->getOutputItem(VEGINDEX_EDR));

   // Refer to the preProcessGranuleSurfaceType function for details about
   // why the Surface Reflectance Ip is being used. 
   preProcessGranuleSurfaceType(&(derivedAlg_->viDataType_.surfReflectIP->qfbyte1[0][0]));

   // Create a data quality notification object for this output item.
   // This new'd memory is deleted by ProCmnAlgorithm::cleanup()
   derivedAlg_->dqnList_.push_back(new ProCmnQualityNotification(
            static_cast<ProCmnAuxDataItem*>(derivedAlg_->getInputItem(VI_DQTT)),
            vegIndexEdrPtr,
            static_cast<ProCmnDQNOutputItem*>(derivedAlg_->getOutputItem(VI_EDR_DQN)),
            VI_EDR_DQN));

   // Create the data quality threshold table test objects
   for(Int32 i = 0; i < derivedAlg_->dqnList_.size(); ++i)
   {
      IngMsdThresholds_DQTT* dqttPtr=0;
      IngMsdThresholds_Entry* dqttEntryPtr=0;
      ProCmnQualityNotification* qnPtr = derivedAlg_->dqnList_.at(i);

      // Note: The memory new'd as addQualityItem parameters are deleted
      // by ProCmnQualityNotification::~ProCmnQualityNotification() 

      // EVI Summary Quality
      qnPtr->addQualityItem(new ProCmnPercentQualityFlag<>(
               0, 
               false, 
               PRO_CMN_OUTPUT,
               &(derivedAlg_->viDataType_.viirsVegIndexIP->qfbyte0[0][0]),
               PRO_CMN_VIIRS_IMAGERY,
               VI_EVI_QUAL_MASK,
               VI_EVI_QUAL_HIGH,
               VI_EVI_QUAL,
               0,
               false,
               &(vegIndexEdrPtr->getQualityValue(VI_EVI_QUAL)),
               false,
               false,
               false,
               &(derivedAlg_->viDataType_.viirsVegIndexIP->TOA_NDVI[0][0])));  

      // NDVI Summary Quality
      qnPtr->addQualityItem(new ProCmnPercentQualityFlag<>(
               0, 
               false, 
               PRO_CMN_OUTPUT,
               &(derivedAlg_->viDataType_.viirsVegIndexIP->qfbyte0[0][0]),
               PRO_CMN_VIIRS_IMAGERY,
               VI_NDVI_QUAL_MASK,
               VI_NDVI_QUAL_HIGH,
               VI_NDVI_QUAL,
               0,
               false,
               &(vegIndexEdrPtr->getQualityValue(VI_NDVI_QUAL)),
               false,
               false,
               false,
               &(derivedAlg_->viDataType_.viirsVegIndexIP->TOA_NDVI[0][0])));  
	  
	  // TOCNDVI Summary Quality
      qnPtr->addQualityItem(new ProCmnPercentQualityFlag<>(
               0, 
               false, 
               PRO_CMN_OUTPUT,
               &(derivedAlg_->viDataType_.viirsVegIndexIP->qfbyte0[0][0]),
               PRO_CMN_VIIRS_IMAGERY,
               VI_TOCNDVI_QUAL_MASK,
               VI_TOCNDVI_QUAL_HIGH,
               VI_TOCNDVI_QUAL,
               0,
               false,
               &(vegIndexEdrPtr->getQualityValue(VI_TOCNDVI_QUAL)),
               false,
               false,
               false,
               &(derivedAlg_->viDataType_.viirsVegIndexIP->TOA_NDVI[0][0])));
			   
      // EVI Exclusion Summary
      qnPtr->addQualityItem(new ProCmnPercentQualityFlag<>(
               0, 
               false, 
               PRO_CMN_OUTPUT,
               eviExclusionSummaryDataPtr_,
               PRO_CMN_VIIRS_IMAGERY,
               DEFAULT_EXCLUSION_MASK,
               DEFAULT_EXCLUSION_MASK,
               VI_EVI_EXCL_SUM,
               0,
               false,
               &(vegIndexEdrPtr->getQualityValue(VI_EVI_EXCL_SUM)),
               false,
               false,
               false,
               &(derivedAlg_->viDataType_.viirsVegIndexIP->TOA_NDVI[0][0])));  

      // NDVI Exclusion Summary
      qnPtr->addQualityItem(new ProCmnPercentQualityFlag<>(
               0, 
               false, 
               PRO_CMN_OUTPUT,
               ndviExclusionSummaryDataPtr_,
               PRO_CMN_VIIRS_IMAGERY,
               DEFAULT_EXCLUSION_MASK,
               DEFAULT_EXCLUSION_MASK,
               VI_NDVI_EXCL_SUM,
               0,
               false,
               &(vegIndexEdrPtr->getQualityValue(VI_NDVI_EXCL_SUM)),
               false,
               false,
               false,
               &(derivedAlg_->viDataType_.viirsVegIndexIP->TOA_NDVI[0][0])));  
		
	  // TOC NDVI Exclusion Summary
      qnPtr->addQualityItem(new ProCmnPercentQualityFlag<>(
               0, 
               false, 
               PRO_CMN_OUTPUT,
               tocndviExclusionSummaryDataPtr_,
               PRO_CMN_VIIRS_IMAGERY,
               DEFAULT_EXCLUSION_MASK,
               DEFAULT_EXCLUSION_MASK,
               VI_TOCNDVI_EXCL_SUM,
               0,
               false,
               &(vegIndexEdrPtr->getQualityValue(VI_TOCNDVI_EXCL_SUM)),
               false,
               false,
               false,
               &(derivedAlg_->viDataType_.viirsVegIndexIP->TOA_NDVI[0][0])));
			   
      // No Land Coverage Quality
      qnPtr->addQualityItem(new ProCmnPercentQualityFlag<>(
               0,
               false,
               PRO_CMN_OUTPUT,
               &(derivedAlg_->landInPixelIndicator_),
               PRO_CMN_DYN_SIZE,
               LAND_IS_IN_PIXEL,
               VI_LAND_ONLY,
               VI_EXCL_NO_LAND,
               1,
               false,
               &(vegIndexEdrPtr->getQualityValue(VI_EXCL_NO_LAND)),
               true,
               false,
               false,
               &(derivedAlg_->viDataType_.viirsVegIndexIP->TOA_NDVI[0][0])));

      // Make sure we retrieved a DQTT from DMS, if the address
      // is zero then continue within the loop.
      if (qnPtr->getDqttItemPtr()->dataPtrUnknown(ProCmnDataItem::DEFAULT_IDX))
      {
         continue;
      }

      // Get the pointer to the DQTT input
      dqttPtr = static_cast<IngMsdThresholds_DQTT *>
         (qnPtr->getDqttItemPtr()->getDataPtr(ProCmnDataItem::DEFAULT_IDX,
                                              callerID(__LINE__)));

      dqttEntryPtr = reinterpret_cast<IngMsdThresholds_Entry*>
         (reinterpret_cast<char*>(dqttPtr) + sizeof(IngMsdThresholds_DQTT));

      for (Int32 j = 0; j < dqttPtr->testCount; j++, dqttEntryPtr++)
      {
         // switch over dqtt test id
         switch(dqttEntryPtr->id)
         {
            // Summary EVI Quality
            case ProCmnVegIndexTest1:
               qnPtr->addQualityItem(new ProCmnPercentQualityFlag<>(
                        dqttEntryPtr->id, 
                        dqttEntryPtr->active, 
                        PRO_CMN_OUTPUT,
                        &(vegIndexEdrPtr->getQualityValue(VI_EVI_QUAL))));   
               break;

               // Summary NDVI Quality
            case ProCmnVegIndexTest2:
               qnPtr->addQualityItem(new ProCmnPercentQualityFlag<>(
                        dqttEntryPtr->id, 
                        dqttEntryPtr->active, 
                        PRO_CMN_OUTPUT,
                        &(vegIndexEdrPtr->getQualityValue(VI_NDVI_QUAL))));   
               break;

			   // Summary TOC NDVI Quality
            case ProCmnVegIndexTest3:
               qnPtr->addQualityItem(new ProCmnPercentQualityFlag<>(
                        dqttEntryPtr->id, 
                        dqttEntryPtr->active, 
                        PRO_CMN_OUTPUT,
                        &(vegIndexEdrPtr->getQualityValue(VI_TOCNDVI_QUAL))));   
               break;
			   
               // EVI Exclusion Summary
            case ProCmnVegIndexTest4:
               qnPtr->addQualityItem(new ProCmnPercentQualityFlag<>(
                        dqttEntryPtr->id, 
                        dqttEntryPtr->active, 
                        PRO_CMN_OUTPUT,
                        &(vegIndexEdrPtr->getQualityValue(VI_EVI_EXCL_SUM))));   
               break;

               // NDVI Exclusion Summary
            case ProCmnVegIndexTest5:
               qnPtr->addQualityItem(new ProCmnPercentQualityFlag<>(
                        dqttEntryPtr->id, 
                        dqttEntryPtr->active, 
                        PRO_CMN_OUTPUT,
                        &(vegIndexEdrPtr->getQualityValue(VI_NDVI_EXCL_SUM))));   
               break;  

			   // TOC NDVI Exclusion Summary
            case ProCmnVegIndexTest6:
               qnPtr->addQualityItem(new ProCmnPercentQualityFlag<>(
                        dqttEntryPtr->id, 
                        dqttEntryPtr->active, 
                        PRO_CMN_OUTPUT,
                        &(vegIndexEdrPtr->getQualityValue(VI_TOCNDVI_EXCL_SUM))));   
               break;
			   
            default:
               std::ostringstream oss;
               oss << "createQualityTests() Encountered unexpected test Id: ";
               oss << dqttEntryPtr->id;
               oss << " from the DQTT.";
               derivedAlg_->infStatusGeneral(INF_STATUSTYPE_PRO_FREE_FORM, 
                       oss.str());  
               break;
         }//Switch 
      }//For Loop
   }//For Loop

}//createQualityTests

//-----------------------------------------------------------------------------
// calculateExclusion() - calculates the exclusion summary
//-----------------------------------------------------------------------------
Int32 ProEdrViirsVIDelegate::calculateExclusion()
{
   // log the begining of the function call
   ProCmnMethodAudit methodAudit(
         this,
         "ProEdrViirsVIDelegate",
         "calculateExclusion",
         derivedAlg_->getMethodContext());

   Int32 status = PRO_SUCCESS;

   // Invoke the helper function to calucate the EVI exclusion
   status = calculateEviExclusion();
   
   if(status != PRO_SUCCESS)
   {
      methodAudit.setRootStatus(status,
            "Error: Calculate EVI Exclusion Failed",
            __FILE__, __LINE__);
      return status;
   }

   // Invoke the helper function to calucate the NDVI exclusion
   status = calculateNdviExclusion();
   
   if(status != PRO_SUCCESS)
   {
      methodAudit.setRootStatus(status,
            "Error: Calculate NDVI Exclusion Failed",
            __FILE__, __LINE__);
      return status;
   }
   
   // Invoke the helper function to calucate the TOC NDVI exclusion
   status = calculateTocNdviExclusion();
   
   if(status != PRO_SUCCESS)
   {
      methodAudit.setRootStatus(status,
            "Error: Calculate TOC NDVI Exclusion Failed",
            __FILE__, __LINE__);
      return status;
   }

   return status;

}

//-----------------------------------------------------------------------------
// calculateEviExclusion() - calculates the EVI exclusion summary
//-----------------------------------------------------------------------------
Int32 ProEdrViirsVIDelegate::calculateEviExclusion()
{
   // log the begining of the function call
   ProCmnMethodAudit methodAudit(
         this,
         "ProEdrViirsVIDelegate",
         "calculateEviExclusion",
         derivedAlg_->getMethodContext());

   Int32 status = PRO_SUCCESS;
   const Int32 totalSize = I_VIIRS_SDR_ROWS * I_VIIRS_SDR_COLS;

   if (eviExclusionSummaryDataPtr_)
   {
      memset(eviExclusionSummaryDataPtr_, 0, 
        (sizeof(UInt8) * totalSize));
   }
   else
   {
      status = PRO_FAIL;

      return status;
   }

   UInt8* qf2_ = &(derivedAlg_->viDataType_.viirsVegIndexIP->qfbyte1[0][0]);
   UInt8* qf3_ = &(derivedAlg_->viDataType_.viirsVegIndexIP->qfbyte2[0][0]);

   UInt8 ccFlag = 0;
   UInt8 lwFlag = 0;
   UInt8 excAotFlag = 0;
   UInt8 excSolZenFlag = 0;

   for (int counter = 0; counter < totalSize; counter++)
   {
      excAotFlag = (*(qf3_ + counter) & VI_AOT_EXCL_MASK);
      excSolZenFlag = (*(qf3_ + counter) & VI_SZA_EXCL_MASK);
      ccFlag = (*(qf2_ + counter) & VI_CLOUD_MASK);
      lwFlag = (*(qf2_ + counter) & VI_LAND_WATER_MASK);
      // Land/Water Background Pixel Flag Indicates
      // "Sea Water" Or "Coastal",
      // AOT > 1, SZA > 85 degrees, Or Cloud Confidence
      // Pixel Does Not Indicate "Confidently Clear".
      if ((lwFlag == VI_SEA_WATER || lwFlag == VI_COASTAL) ||
          excAotFlag == VI_AOT_GTR_1 ||
          excSolZenFlag == VI_SZA_GTR_85 ||
          ccFlag != VI_CNFD_CLEAR)
      {
         *(eviExclusionSummaryDataPtr_ + counter) = 1;
      }
   }

   return status;

}

//-----------------------------------------------------------------------------
// calculateNdviExclusion() - calculates the NDVI exclusion summary
//-----------------------------------------------------------------------------
Int32 ProEdrViirsVIDelegate::calculateNdviExclusion()
{
   // log the begining of the function call
   ProCmnMethodAudit methodAudit(
         this,
         "ProEdrViirsVIDelegate",
         "calculateNdviExclusion",
         derivedAlg_->getMethodContext());

   Int32 status = PRO_SUCCESS;
   const Int32 totalSize = I_VIIRS_SDR_ROWS * I_VIIRS_SDR_COLS;

   if (ndviExclusionSummaryDataPtr_)
   {
      memset(ndviExclusionSummaryDataPtr_, 0, 
        (sizeof(UInt8) * totalSize));
   }
   else
   {
      status = PRO_FAIL;

      return status;
   }

   UInt8* qf2_ = &(derivedAlg_->viDataType_.viirsVegIndexIP->qfbyte1[0][0]);
   UInt8* qf3_ = &(derivedAlg_->viDataType_.viirsVegIndexIP->qfbyte2[0][0]);

   UInt8 ccFlag = 0;
   UInt8 lwFlag = 0;
   UInt8 excSolZenFlag = 0;

   for (int counter = 0; counter < totalSize; counter++)
   {
      excSolZenFlag = (*(qf3_ + counter) & VI_SZA_EXCL_MASK);
      ccFlag = (*(qf2_ + counter) & VI_CLOUD_MASK);
      lwFlag = (*(qf2_ + counter) & VI_LAND_WATER_MASK);
      // Land/Water Background Pixel Flag Indicates
      // "Sea Water" Or "Coastal",
      // SZA > 85 degrees, Or Cloud Confidence
      // Pixel Does Not Indicate "Confidently Clear".
      if ((lwFlag == VI_SEA_WATER || lwFlag == VI_COASTAL) ||
          excSolZenFlag == VI_SZA_GTR_85 ||
          ccFlag != VI_CNFD_CLEAR)
      {
         *(ndviExclusionSummaryDataPtr_ + counter) = 1;
      }
   }

   return status;
}

//-----------------------------------------------------------------------------
// calculateTocNdviExclusion() - calculates the TOC NDVI exclusion summary
//-----------------------------------------------------------------------------
Int32 ProEdrViirsVIDelegate::calculateTocNdviExclusion()
{
   // log the begining of the function call
   ProCmnMethodAudit methodAudit(
         this,
         "ProEdrViirsVIDelegate",
         "calculateTocNdviExclusion",
         derivedAlg_->getMethodContext());

   Int32 status = PRO_SUCCESS;
   const Int32 totalSize = I_VIIRS_SDR_ROWS * I_VIIRS_SDR_COLS;

   if (tocndviExclusionSummaryDataPtr_)
   {
      memset(tocndviExclusionSummaryDataPtr_, 0, 
        (sizeof(UInt8) * totalSize));
   }
   else
   {
      status = PRO_FAIL;

      return status;
   }

   UInt8* qf2_ = &(derivedAlg_->viDataType_.viirsVegIndexIP->qfbyte1[0][0]);
   UInt8* qf3_ = &(derivedAlg_->viDataType_.viirsVegIndexIP->qfbyte2[0][0]);

   UInt8 ccFlag = 0;
   UInt8 lwFlag = 0;
   UInt8 excSolZenFlag = 0;

   for (int counter = 0; counter < totalSize; counter++)
   {
      excSolZenFlag = (*(qf3_ + counter) & VI_SZA_EXCL_MASK);
      ccFlag = (*(qf2_ + counter) & VI_CLOUD_MASK);
      lwFlag = (*(qf2_ + counter) & VI_LAND_WATER_MASK);
      // Land/Water Background Pixel Flag Indicates
      // "Sea Water" Or "Coastal",
      // SZA > 85 degrees, Or Cloud Confidence
      // Pixel Does Not Indicate "Confidently Clear".
      if ((lwFlag == VI_SEA_WATER || lwFlag == VI_COASTAL) ||
          excSolZenFlag == VI_SZA_GTR_85 ||
          ccFlag != VI_CNFD_CLEAR)
      {
         *(tocndviExclusionSummaryDataPtr_ + counter) = 1;
      }
   }

   return status;
}
//--------------------------------------------------------------------------
// Determine the land/water flag for every non-trim pixel in the granule
// to determine if there are land pixels present in the granule.
//
// Note: One of the very first implementations of this function did use the
// quality flags from the VIIRS-VI-EDR. However, we discovered that when
// the moderate information in the VIIRS-Surf-Refl-IP is aggregated up
// to the VIIRS-VI-EDR, which is at the imagery resolution, the
// aggregation misses (not intentionally) certain non-trim pixels that
// this function would end up examining. So, we had two options - we
// could either use the non-trim-moderate-pixel-to-imagery-resolution loop
// when examining the VIIRS-VI-EDR. Or we could just examine the 
// VIIRS-Surf-Refl-IP which is the basis for many of the quality flags in
// the VIIRS-VI-EDR. 
//--------------------------------------------------------------------------
void ProEdrViirsVIDelegate::preProcessGranuleSurfaceType(const UInt8* qf2)
{
   ProCmnViirsTrimTable* trimTable = ProCmnViirsTrimTable::getInstance();

   UInt32 totalSize = M_VIIRS_SDR_ROWS * M_VIIRS_SDR_COLS;
   UInt32 offset = 0;
   
   /**
    * Account for short granules.  This must be done because the default
    * initialization state of this QF in SR is '0' meaning a pixel would
    * be marked as 'land' if we moved across all scans instead of 
    * just active scans.
    */
   Int32 maxRow = derivedAlg_->viDataType_.geo->act_scans * M_DETECTORS; 
   Int32 colSize = M_VIIRS_SDR_COLS;
   Int32 colMin = 0;
   Int32 colMax = 0;

   UInt8 landWaterFlag = 0x00;
   derivedAlg_->landInPixelIndicator_ = 0x00;

   for (Int32 rowCounter = 0; rowCounter < maxRow; rowCounter++)
   {
      colMin = trimTable->getModLowerBound(rowCounter);
      colMax = trimTable->getModUpperBound(rowCounter);

      for (Int32 colCounter = colMin; colCounter <= colMax; colCounter++)
      {
         offset = (rowCounter * colSize) + colCounter;
         
         landWaterFlag = (*(qf2 + offset) & VI_LAND_WATER_MASK);
         if (landWaterFlag == VI_LAND_DESERT ||
             landWaterFlag == VI_LAND_NO_DESERT)
         {
            derivedAlg_->landInPixelIndicator_ = LAND_IS_IN_PIXEL;
            // We Found A Pixel That Contains Land, So It
            // Is Not Possible For The Granule To Not Contain
            // Land. There Is No Reason For Us To Continue
            // Searching For Pixels That Contain Land.
            return;
         }
      } 
   }
}

//-----------------------------------------------------------------------------
// Returns the identity string.
//-----------------------------------------------------------------------------
/*virtual*/ const std::string& ProEdrViirsVIDelegate::getIdentity() const
{
    return identity_;
}

//-----------------------------------------------------------------------------
// my caller identity
//-----------------------------------------------------------------------------
const ProCmnCallerID* ProEdrViirsVIDelegate::callerID(int linenum) const
{
    // abstract meaning of const
    ((ProEdrViirsVIDelegate*)this)->myCallerID_.setLineNumber(linenum);

    return &myCallerID_;
}

//-----------------------------------------------------------------------------
// Update outputs for shell processing
//-----------------------------------------------------------------------------
Int32 ProEdrViirsVIDelegate::produceShellOutputs(ProCmnCallerID *caller)
{
    ProCmnMethodAudit methodAudit(
         this,
         "ProEdrViirsVIDelegate",
         "produceShellOutputs",
         derivedAlg_->getMethodContext());

    Int32 status = PRO_SUCCESS;

    status = derivedAlg_->produceShellOutputs(caller);

    return status;
}

