/* BEGIN COPYRIGHT */
/* END COPYRIGHT */
/* BEGIN GOVERNMENT_RIGHTS */
/***********************************************************************
 * UNLIMITED RIGHTS
 * This material may be reproduced by or on behalf of the
 * U.S. Government pursuant to the copyright license under the DFAR
 * 252.227-7014 which was made part of the above contract.
 ***********************************************************************/
/* END GOVERNMENT_RIGHTS */
/* BEGIN ITAR */
/* END ITAR */

/* BEGIN CATEGORY */
/*******************************************************************************
 *  CATEGORY: 1
 *  (<ProEdrViirsVIGbl.h>)
 *
 *  Refer to Software Standards and Practices Manual (SSPM) for coding standards
 *
 *******************************************************************************/
/* END CATEGORY */

/*******************************************************************************
 *
 *  NAME:  ProEdrViirsVIGbl.h 
 * 
 *  DESCRIPTION: Defines the data structure for the Viirs Vegetation Index EDR
 *             
 *  REFERENCES: 
 *  Y2400  - Vegetation Index Algorithm Theoretical Basis Document
 *  Y2474  - Land Module Level Software Architecture Document
 *  Y2499  - VIIRS Vegetation Index Unit Level Detailed Design Document
 *  Y10883 - VIIRS Land Module Data Dictionary
 *  D36951 - Operational Algorithm Document for the VIIRS Vegetation Index EDR
 *           Software
 *
 *  LIMITATIONS: none
 * 
 *  REVISION/EVENT HISTORY:
 *  DATE        PR#      AUTHOR            Build    DESCRIPTION
 *  ---------   ---      ------            -----    -----------
 *  12JAN2004            mhoover           1.2      Initial Version
 *  31MAR2004   1778     mhoover           1.2      modified header definition
 *  25JUL2006            C. Erkes          1.5      Updated header to include 
 *                                                  quality flags and bit 
 *                                                  descriptions
 *  17OCT2006            C. Erkes          1.5      Updated output structure to
 *                                                  include scale and offset 
 *                                                  fields
 *  06MAY2007            B. Henderson      1.5      Added full structure
 *  10DEC2007   16155    T. Gardner        1.5      Corrected VI_LAND_WATER_MASK
 *  30NOV2007   15892    B. Le             1.5.x.1  ECR A-103, EDRPR 1.8 CP 3 updates -
 *                                                  Format changes for CDFCB-X                 
 *                                                  compliance - Remove references
 *                                                  to granuleLevelFlagByte0
 *  09DEC2009   21642    J. Gibbs          SenChar  Updated quality summary
 *                                                  masks
 *  21JAN2011            D. Doumakes                Updated for ADL:  removed
 *                                                  autogenerated types
 *  02FEB2014   37359    W. Dare            Mx8.4   474-CCR-13-1218, QF update
 * 
 *  NOTES:
 * 
 *******************************************************************************/

#ifndef ProEdrViirsVIGbl_h
#define ProEdrViirsVIGbl_h

#include <Typedefs.h>
#include <viirs_SDRf.h>

/********************************************************************
 * VI Quality Flag Masks                                            *
 ********************************************************************/
// Byte 0
// Bit 0 - Overall NDVI Quality 
const UInt8 VI_NDVI_QUAL_MASK          = 0x01;
const UInt8 VI_NDVI_QUAL_LOW           = 0x00;
const UInt8 VI_NDVI_QUAL_HIGH          = 0x01;

// Bit 1 - Overall EVI Quality
const UInt8 VI_EVI_QUAL_MASK           = 0x02;
const UInt8 VI_EVI_QUAL_LOW            = 0x00;
const UInt8 VI_EVI_QUAL_HIGH           = 0x02;

// Bit 2 - I1 TOA Reflectance
const UInt8 VI_I1_TOA_REFL_AVAIL_MASK  = 0x04;
const UInt8 VI_I1_TOA_REFL_AVAIL       = 0x00;
const UInt8 VI_I1_TOA_REFL_NOT_AVAIL   = 0x04;

// Bit 3 - I2 TOA Reflectance
const UInt8 VI_I2_TOA_REFL_AVAIL_MASK  = 0x08;
const UInt8 VI_I2_TOA_REFL_AVAIL       = 0x00;
const UInt8 VI_I2_TOA_REFL_NOT_AVAIL   = 0x08;

// Bit 4 - I1 Surface Reflectance 
const UInt8 VI_I1_SURF_REFL_AVAIL_MASK = 0x10;
const UInt8 VI_I1_SURF_REFL_AVAIL      = 0x00;
const UInt8 VI_I1_SURF_REFL_NOT_AVAIL  = 0x10;

// Bit 5 - I2 Surface Reflectance
const UInt8 VI_I2_SURF_REFL_AVAIL_MASK = 0x20;
const UInt8 VI_I2_SURF_REFL_AVAIL      = 0x00;
const UInt8 VI_I2_SURF_REFL_NOT_AVAIL  = 0x20;

// Bit 6 - M3 Surface Reflectance
const UInt8 VI_M3_SURF_REFL_AVAIL_MASK = 0x40;
const UInt8 VI_M3_SURF_REFL_AVAIL      = 0x00;
const UInt8 VI_M3_SURF_REFL_NOT_AVAIL  = 0x40;

// Bit 7 - EVI Range
const UInt8 VI_EVI_RANGE_MASK          = 0x80;
const UInt8 VI_EVI_IN_RANGE            = 0x00;
const UInt8 VI_EVI_OUT_OF_RANGE        = 0x80;

// Byte 1
// Bit 0, 1, 2 - LAND/WATER
const UInt8 VI_LAND_WATER_MASK         = 0x07;
const UInt8 VI_LAND_DESERT             = 0x00;
const UInt8 VI_LAND_NO_DESERT          = 0x01;
const UInt8 VI_INLAND_WATER            = 0x02;
const UInt8 VI_SEA_WATER               = 0x03;
const UInt8 VI_COASTAL                 = 0x05;

const UInt8 VI_LAND_ONLY_MASK          = 0x06;
const UInt8 VI_LAND_ONLY               = 0x00;

// Bit 3, 4 - Cloud Confidence
const UInt8 VI_CLOUD_MASK              = 0x18;
const UInt8 VI_CNFD_CLEAR              = 0x00;
const UInt8 VI_PROB_CLEAR              = 0x08;
const UInt8 VI_PROB_CLOUDY             = 0x10;
const UInt8 VI_CNFD_CLOUDY             = 0x18;

// Bit 5, 6 - Sun Glint
const UInt8 VI_SUN_GLINT_MASK             = 0x60;
const UInt8 VI_NO_SUN_GLINT               = 0x00;
const UInt8 VI_GEO_BASED_SUN_GLINT        = 0x20;
const UInt8 VI_WIND_SPEED_BASED_SUN_GLINT = 0x40;
const UInt8 VI_GEO_WIND_BASED_SUN_GLINT   = 0x60;

// Bit 7 - Thin Cirrus
const UInt8 VI_THIN_CIR_REFL_MASK      = 0x80;
const UInt8 VI_THIN_CIR_NO_CLOUD       = 0x00;
const UInt8 VI_THIN_CIR_CLOUD          = 0x80;

// Byte 2
// Bit 1 - SZA Stratification
const UInt8 VI_SZA_STRAT_MASK          = 0x01;
const UInt8 VI_SZA_LESS_65_DEG         = 0x00;
const UInt8 VI_SZA_GTR_65_LESS_85      = 0x01;

// Bit 2 - AOT Exclusion 
const UInt8 VI_AOT_EXCL_MASK           = 0x02;
const UInt8 VI_AOT_LESS_1              = 0x00;
const UInt8 VI_AOT_GTR_1               = 0x02;

// Bit 3 - SZA Greater 85 Degrees
const UInt8 VI_SZA_EXCL_MASK           = 0x04;
const UInt8 VI_SZA_LESS_85             = 0x00;
const UInt8 VI_SZA_GTR_85              = 0x04;

// New additions by LPEATE to utlize the spare 5 bits
// with new QF flags from SRIP (qf6)
// Snow/ice:            bit 3 of qf2 from bit 0 of qf6
const UInt8  VI_SNOW_ICE_MASK        = 0x08;
const UInt8  SR_SNOW_ICE_MASK        = 0x01;
// Adjacent to clouds:  bit 4 of qf2 from bit 1 of qf6
const UInt8  VI_ADJ_CLOUDS_MASK      = 0X10;
const UInt8  SR_ADJ_CLOUDS_MASK      = 0x02;
// Aerosol quantity: bits 5-6 of qf2 from bits 2-3 of qf6
const UInt8  VI_AERO_QUANTITY_MASK   = 0x60;
const UInt8  SR_AERO_QUANTITY_MASK   = 0x0C;
// Cloud shadow:        bit 7 of qf2 from bit 3 of qf1
const UInt8  VI_CLD_SHADOW_MASK      = 0x80;
// End of LPEATE addition

// Byte 3
// Bit 0 - Overall TOC NDVI Quality 
const UInt8 VI_TOCNDVI_QUAL_MASK          = 0x01;
const UInt8 VI_TOCNDVI_QUAL_LOW           = 0x00;
const UInt8 VI_TOCNDVI_QUAL_HIGH          = 0x01;

// Mask for Granule level quality flag
const UInt8 VI_EXCLUSION_MASK          = 0x01;
const UInt8 VI_NO_LAND_MASK            = 0x01;

/********************************************************************
 * VI Quality Flag Shifts                                           *
 ********************************************************************/
// Byte 0
const UInt32 VI_EVI_QUAL_SHIFT           = 1;
const UInt32 VI_I1_TOA_REFL_AVAIL_SHIFT  = 2;
const UInt32 VI_I2_TOA_REFL_AVAIL_SHIFT  = 3;
const UInt32 VI_I1_SURF_REFL_AVAIL_SHIFT = 4;
const UInt32 VI_I2_SURF_REFL_AVAIL_SHIFT = 5;
const UInt32 VI_M3_SURF_REFL_AVAIL_SHIFT = 6;
const UInt32 VI_EVI_RANGE_SHIFT          = 7;

// Byte 1
const UInt32 VI_CLOUD_SHIFT              = 3;
const UInt32 VI_SUN_GLINT_SHIFT          = 5;
const UInt32 VI_THIN_CIR_REFL_SHIFT      = 7;

// Byte 2
const UInt32 VI_AOT_EXCL_SHIFT           = 1;
const UInt32 VI_SZA_EXCL_SHIFT           = 2;

// New LPEATE additions to utilize the spare 5 bits
const UInt32 VI_SNOW_ICE_SHIFT       = 3;
const UInt32 SR_SNOW_ICE_SHIFT       = 0;
const UInt32 VI_ADJ_CLOUDS_SHIFT     = 4;
const UInt32 SR_ADJ_CLOUDS_SHIFT     = 1;
const UInt32 VI_AERO_QUANTITY_SHIFT  = 5;
const UInt32 SR_AERO_QUANTITY_SHIFT  = 2;
const UInt32 VI_CLD_SHADOW_SHIFT     = 7;

// bit definitions
const UInt8 VI_CONF_CLEAR               = 0x03;
const UInt8 VI_NO_GLINT                 = 0x00;



#endif 
